@@ -4,14 +4,16 @@
  * SPDX-License-Identifier: LGPL-2.1+
  */
 
+#include "config.h"
 #include "hdy-combo-row.h"
 
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include "hdy-list-box.h"
+#include "hdy-style-private.h"
 
 /**
  * SECTION:hdy-combo-row
- * @short_description: A #GtkListBox row used to choose from a list of items
+ * @short_description: A #GtkListBox row used to choose from a list of items.
  * @Title: HdyComboRow
  *
  * The #HdyComboRow widget allows the user to choose from a list of valid
@@ -29,35 +31,50 @@
  * This was mostly inspired by code from the display panel from GNOME Settings.
  */
 
+typedef struct
+{
+  HdyComboRowGetNameFunc func;
+  gpointer func_data;
+  GDestroyNotify func_data_destroy;
+} HdyComboRowGetName;
+
 typedef struct
 {
   GtkBox *current;
   GtkImage *image;
   GtkListBox *list;
   GtkPopover *popover;
-  gint selected_position;
+  gint selected_index;
+  gboolean use_subtitle;
+  HdyComboRowGetName *get_name;
 
   GListModel *bound_model;
   GtkListBoxCreateWidgetFunc create_list_widget_func;
   GtkListBoxCreateWidgetFunc create_current_widget_func;
   gpointer create_widget_func_data;
+  /* This is owned by create_widget_func_data, which is ultimately owned by the
+   * list box, and hence should not be destroyed manually.
+   */
+  HdyComboRowGetName *get_name_internal;
 } HdyComboRowPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (HdyComboRow, hdy_combo_row, HDY_TYPE_ACTION_ROW)
 
-typedef struct
-{
-  HdyComboRowGetNameFunc get_name_func;
-  gpointer get_name_func_data;
-  GDestroyNotify get_name_func_data_destroy;
-} HdyComboRowCreateLabelData;
+enum {
+  PROP_0,
+  PROP_SELECTED_INDEX,
+  PROP_USE_SUBTITLE,
+  LAST_PROP,
+};
+
+static GParamSpec *props[LAST_PROP];
 
 static GtkWidget *
 create_list_label (gpointer item,
                    gpointer user_data)
 {
-  HdyComboRowCreateLabelData *data = user_data;
-  g_autofree gchar *name = data->get_name_func (item, data->get_name_func_data);
+  HdyComboRowGetName *get_name = (HdyComboRowGetName *) user_data;
+  g_autofree gchar *name = get_name->func (item, get_name->func_data);
 
   return g_object_new (GTK_TYPE_LABEL,
                        "ellipsize", PANGO_ELLIPSIZE_END,
@@ -74,11 +91,11 @@ static GtkWidget *
 create_current_label (gpointer item,
                       gpointer user_data)
 {
-  HdyComboRowCreateLabelData *data = user_data;
+  HdyComboRowGetName *get_name = (HdyComboRowGetName *) user_data;
   g_autofree gchar *name = NULL;
 
-  if (data->get_name_func)
-    name = data->get_name_func (item, data->get_name_func_data);
+  if (get_name->func)
+    name = get_name->func (item, get_name->func_data);
 
   return g_object_new (GTK_TYPE_LABEL,
                        "ellipsize", PANGO_ELLIPSIZE_END,
@@ -91,79 +108,91 @@ create_current_label (gpointer item,
 }
 
 static void
-create_label_data_free (gpointer user_data)
+get_name_free (HdyComboRowGetName *get_name)
 {
-  HdyComboRowCreateLabelData *data = user_data;
-
-  if (data == NULL)
+  if (get_name == NULL)
     return;
 
-  if (data->get_name_func_data_destroy)
-    data->get_name_func_data_destroy (data->get_name_func_data);
-  data->get_name_func_data = NULL;
-  data->get_name_func_data_destroy = NULL;
+  if (get_name->func_data_destroy)
+    get_name->func_data_destroy (get_name->func_data);
+  get_name->func = NULL;
+  get_name->func_data = NULL;
+  get_name->func_data_destroy = NULL;
 
-  g_free (data);
+  g_free (get_name);
 }
 
 static void
 update (HdyComboRow *self)
 {
   HdyComboRowPrivate *priv = hdy_combo_row_get_instance_private (self);
-  gpointer item;
+  g_autoptr(GObject) item = NULL;
+  g_autofree gchar *name = NULL;
   GtkWidget *widget;
 
+  gtk_widget_set_visible (GTK_WIDGET (priv->current), !priv->use_subtitle);
   gtk_container_foreach (GTK_CONTAINER (priv->current), (GtkCallback) gtk_widget_destroy, NULL);
 
   if (priv->bound_model == NULL || g_list_model_get_n_items (priv->bound_model) == 0) {
     gtk_widget_set_sensitive (GTK_WIDGET (self), FALSE);
-    priv->selected_position = -1;
+    g_assert (priv->selected_index == -1);
 
     return;
   }
 
-  if (priv->selected_position == -1)
-    priv->selected_position = 0;
+  g_assert (priv->selected_index >= 0 && priv->selected_index <= g_list_model_get_n_items (priv->bound_model));
 
   gtk_widget_set_sensitive (GTK_WIDGET (self), TRUE);
 
-  item = g_list_model_get_item (priv->bound_model, priv->selected_position);
-  widget = priv->create_current_widget_func (item, priv->create_widget_func_data);
-  gtk_container_add (GTK_CONTAINER (priv->current), widget);
+  item = g_list_model_get_item (priv->bound_model, priv->selected_index);
+  if (priv->use_subtitle) {
+    if (priv->get_name != NULL && priv->get_name->func)
+      name = priv->get_name->func (item, priv->get_name->func_data);
+    else if (priv->get_name_internal != NULL && priv->get_name_internal->func)
+      name = priv->get_name_internal->func (item, priv->get_name_internal->func_data);
+    hdy_action_row_set_subtitle (HDY_ACTION_ROW (self), name);
+  }
+  else {
+    widget = priv->create_current_widget_func (item, priv->create_widget_func_data);
+    gtk_container_add (GTK_CONTAINER (priv->current), widget);
+  }
 }
 
 static void
 bound_model_changed (GListModel *list,
-                     guint       position,
+                     guint       index,
                      guint       removed,
                      guint       added,
                      gpointer    user_data)
 {
+  gint new_idx;
   HdyComboRow *self = HDY_COMBO_ROW (user_data);
   HdyComboRowPrivate *priv = hdy_combo_row_get_instance_private (self);
 
-  if (priv->selected_position < position)
+  /* Selection is in front of insertion/removal point, nothing to do */
+  if (priv->selected_index > 0 && priv->selected_index < index)
     return;
 
-  if (priv->selected_position < position + removed) {
-    priv->selected_position = -1;
-
-    return;
+  if (priv->selected_index < index + removed) {
+    /* The item selected item was removed (or none is selected) */
+    new_idx = -1;
+  } else {
+    /* The item selected item was behind the insertion/removal */
+    new_idx = priv->selected_index + added - removed;
   }
 
-  priv->selected_position += added;
+  /* Select the first item if none is selected. */
+  if (new_idx == -1 && g_list_model_get_n_items (list) > 0)
+    new_idx = 0;
 
-  update (self);
+  hdy_combo_row_set_selected_index (self, new_idx);
 }
 
 static void
 row_activated_cb (HdyComboRow   *self,
                   GtkListBoxRow *row)
 {
-  HdyComboRowPrivate *priv = hdy_combo_row_get_instance_private (self);
-
-  priv->selected_position = gtk_list_box_row_get_index (row);
-  update (self);
+  hdy_combo_row_set_selected_index (self, gtk_list_box_row_get_index (row));
 }
 
 static void
@@ -185,24 +214,66 @@ destroy_model (HdyComboRow *self)
     g_signal_handlers_disconnect_by_func (priv->bound_model, bound_model_changed, self);
 
     /* Destroy the model and the user data. */
-    gtk_list_box_bind_model (priv->list, NULL, NULL, NULL, NULL);
+    if (priv->list)
+      gtk_list_box_bind_model (priv->list, NULL, NULL, NULL, NULL);
 
     priv->bound_model = NULL;
     priv->create_list_widget_func = NULL;
     priv->create_current_widget_func = NULL;
     priv->create_widget_func_data = NULL;
-
   }
 }
 
 static void
-hdy_combo_row_finalize (GObject *object)
+hdy_combo_row_get_property (GObject    *object,
+                            guint       prop_id,
+                            GValue     *value,
+                            GParamSpec *pspec)
 {
   HdyComboRow *self = HDY_COMBO_ROW (object);
 
-  destroy_model (self);
+  switch (prop_id) {
+  case PROP_SELECTED_INDEX:
+    g_value_set_int (value, hdy_combo_row_get_selected_index (self));
+    break;
+  case PROP_USE_SUBTITLE:
+    g_value_set_boolean (value, hdy_combo_row_get_use_subtitle (self));
+    break;
+  default:
+    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+  }
+}
 
-  G_OBJECT_CLASS (hdy_combo_row_parent_class)->finalize (object);
+static void
+hdy_combo_row_set_property (GObject      *object,
+                            guint         prop_id,
+                            const GValue *value,
+                            GParamSpec   *pspec)
+{
+  HdyComboRow *self = HDY_COMBO_ROW (object);
+
+  switch (prop_id) {
+  case PROP_SELECTED_INDEX:
+    hdy_combo_row_set_selected_index (self, g_value_get_int (value));
+    break;
+  case PROP_USE_SUBTITLE:
+    hdy_combo_row_set_use_subtitle (self, g_value_get_boolean (value));
+    break;
+  default:
+    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+  }
+}
+
+static void
+hdy_combo_row_dispose (GObject *object)
+{
+  HdyComboRow *self = HDY_COMBO_ROW (object);
+  HdyComboRowPrivate *priv = hdy_combo_row_get_instance_private (self);
+
+  destroy_model (self);
+  g_clear_pointer (&priv->get_name, get_name_free);
+
+  G_OBJECT_CLASS (hdy_combo_row_parent_class)->dispose (object);
 }
 
 typedef struct {
@@ -253,14 +324,51 @@ hdy_combo_row_class_init (HdyComboRowClass *klass)
   GtkContainerClass *container_class = GTK_CONTAINER_CLASS (klass);
   HdyActionRowClass *row_class = HDY_ACTION_ROW_CLASS (klass);
 
-  object_class->finalize = hdy_combo_row_finalize;
+  object_class->get_property = hdy_combo_row_get_property;
+  object_class->set_property = hdy_combo_row_set_property;
+  object_class->dispose = hdy_combo_row_dispose;
 
   container_class->forall = hdy_combo_row_forall;
 
   row_class->activate = hdy_combo_row_activate;
 
+  /**
+   * HdyComboRow:selected-index:
+   *
+   * The index of the selected item in its #GListModel.
+   *
+   * Since: 0.0.7
+   */
+  props[PROP_SELECTED_INDEX] =
+      g_param_spec_int ("selected-index",
+                        _("Selected index"),
+                        _("The index of the selected item"),
+                        -1, G_MAXINT, -1,
+                        G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+
+  /**
+   * HdyComboRow:use-subtitle:
+   *
+   * %TRUE to set the current value as the subtitle.
+   *
+   * If you use a custom widget creation function, you will need to give @self
+   * a name conversion closure with hdy_combo_row_set_get_name_func().
+   *
+   * If %TRUE, you should not access HdyActionRow:subtitle.
+   *
+   * Since: 0.0.10
+   */
+  props[PROP_USE_SUBTITLE] =
+    g_param_spec_boolean ("use-subtitle",
+                          _("Use subtitle"),
+                          _("Set the current value as the subtitle"),
+                          FALSE,
+                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+
+  g_object_class_install_properties (object_class, LAST_PROP, props);
+
   gtk_widget_class_set_template_from_resource (widget_class,
-                                               "/sm/puri/handy/dialer/ui/hdy-combo-row.ui");
+                                               "/sm/puri/handy/ui/hdy-combo-row.ui");
   gtk_widget_class_bind_template_child_private (widget_class, HdyComboRow, current);
   gtk_widget_class_bind_template_child_private (widget_class, HdyComboRow, image);
   gtk_widget_class_bind_template_child_private (widget_class, HdyComboRow, list);
@@ -273,12 +381,10 @@ list_init (HdyComboRow *self)
   HdyComboRowPrivate *priv = hdy_combo_row_get_instance_private (self);
   g_autoptr (GtkCssProvider) provider = gtk_css_provider_new ();
 
-  /* This makes the list's background transparent. */
-  gtk_css_provider_load_from_data (GTK_CSS_PROVIDER (provider),
-                                   "list { border-style: none; background-color: transparent; }", -1, NULL);
+  gtk_css_provider_load_from_resource (provider, "/sm/puri/handy/style/hdy-combo-row-list.css");
   gtk_style_context_add_provider (gtk_widget_get_style_context (GTK_WIDGET (priv->list)),
                                   GTK_STYLE_PROVIDER (provider),
-                                  GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+                                  HDY_STYLE_PROVIDER_PRIORITY);
 }
 
 static void
@@ -290,7 +396,7 @@ hdy_combo_row_init (HdyComboRow *self)
 
   list_init (self);
 
-  priv->selected_position = -1;
+  priv->selected_index = -1;
 
   gtk_list_box_set_header_func (priv->list, hdy_list_box_separator_header, NULL, NULL);
   g_signal_connect_object (priv->list, "row-activated", G_CALLBACK (gtk_widget_hide),
@@ -342,9 +448,9 @@ hdy_combo_row_get_model (HdyComboRow *self)
  * hdy_combo_row_bind_model:
  * @self: a #HdyComboRow
  * @model: (nullable): the #GListModel to be bound to @self
- * @create_list_widget_func: (nullable): a function that creates widgets for
- *   items to display in the list, or %NULL in case you also passed %NULL as
- *   @model
+ * @create_list_widget_func: (nullable) (scope call): a function that creates
+ *   widgets for items to display in the list, or %NULL in case you also passed
+ *   %NULL as @model
  * @create_current_widget_func: (nullable) (scope call): a function that creates
  *   widgets for items to display as the seleted item, or %NULL in case you also
  *   passed %NULL as @model
@@ -382,11 +488,12 @@ hdy_combo_row_bind_model (HdyComboRow                *self,
   destroy_model (self);
 
   gtk_container_foreach (GTK_CONTAINER (priv->current), (GtkCallback) gtk_widget_destroy, NULL);
-  priv->selected_position = -1;
+  priv->selected_index = -1;
 
   if (model == NULL) {
     update (self);
 
+    g_object_notify_by_pspec (G_OBJECT (self), props[PROP_SELECTED_INDEX]);
     return;
   }
 
@@ -400,7 +507,12 @@ hdy_combo_row_bind_model (HdyComboRow                *self,
 
   g_signal_connect (priv->bound_model, "items-changed", G_CALLBACK (bound_model_changed), self);
 
+  if (g_list_model_get_n_items (priv->bound_model) > 0)
+    priv->selected_index = 0;
+
   update (self);
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_SELECTED_INDEX]);
 }
 
 /**
@@ -432,18 +544,18 @@ hdy_combo_row_bind_name_model (HdyComboRow            *self,
                                gpointer                user_data,
                                GDestroyNotify          user_data_free_func)
 {
-  HdyComboRowCreateLabelData *data;
+  HdyComboRowPrivate *priv = hdy_combo_row_get_instance_private (self);
 
   g_return_if_fail (HDY_IS_COMBO_ROW (self));
   g_return_if_fail (model == NULL || G_IS_LIST_MODEL (model));
   g_return_if_fail (model == NULL || get_name_func != NULL);
 
-  data = g_new0 (HdyComboRowCreateLabelData, 1);
-  data->get_name_func = get_name_func;
-  data->get_name_func_data = user_data;
-  data->get_name_func_data_destroy = user_data_free_func;
+  priv->get_name_internal = g_new0 (HdyComboRowGetName, 1);
+  priv->get_name_internal->func = get_name_func;
+  priv->get_name_internal->func_data = user_data;
+  priv->get_name_internal->func_data_destroy = user_data_free_func;
 
-  hdy_combo_row_bind_model (self, model, create_list_label, create_current_label, data, create_label_data_free);
+  hdy_combo_row_bind_model (self, model, create_list_label, create_current_label, priv->get_name_internal, (GDestroyNotify) get_name_free);
 }
 
 /**
@@ -487,12 +599,157 @@ hdy_combo_row_set_for_enum (HdyComboRow                     *self,
 
   enum_class = g_type_class_ref (enum_type);
   for (i = 0; i < enum_class->n_values; i++)
-    g_list_store_append (store, hdy_enum_value_object_new (&enum_class->values[i]));
+    {
+      g_autoptr(HdyEnumValueObject) obj = hdy_enum_value_object_new (&enum_class->values[i]);
+
+      g_list_store_append (store, obj);
+    }
 
   hdy_combo_row_bind_name_model (self, G_LIST_MODEL (store), (HdyComboRowGetNameFunc) get_name_func, user_data, user_data_free_func);
   g_type_class_unref (enum_class);
 }
 
+/**
+ * hdy_combo_row_get_selected_index:
+ * @self: a #GtkListBoxRow
+ *
+ * Gets the index of the selected item in its #GListModel.
+ *
+ * Returns: the index of the selected item, or -1 if no item is selected
+ *
+ * Since: 0.0.7
+ */
+gint
+hdy_combo_row_get_selected_index (HdyComboRow *self)
+{
+  HdyComboRowPrivate *priv;
+
+  g_return_val_if_fail (HDY_IS_COMBO_ROW (self), -1);
+
+  priv = hdy_combo_row_get_instance_private (self);
+
+  return priv->selected_index;
+}
+
+/**
+ * hdy_combo_row_set_selected_index:
+ * @self: a #HdyComboRow
+ * @selected_index: the index of the selected item
+ *
+ * Sets the index of the selected item in its #GListModel.
+ *
+ * Since: 0.0.7
+ */
+void
+hdy_combo_row_set_selected_index (HdyComboRow *self,
+                                  gint         selected_index)
+{
+  HdyComboRowPrivate *priv;
+
+  g_return_if_fail (HDY_IS_COMBO_ROW (self));
+  g_return_if_fail (selected_index >= -1);
+
+  priv = hdy_combo_row_get_instance_private (self);
+
+  g_return_if_fail (selected_index >= 0 || priv->bound_model == NULL || g_list_model_get_n_items (priv->bound_model) == 0);
+  g_return_if_fail (selected_index == -1 || (priv->bound_model != NULL && selected_index < g_list_model_get_n_items (priv->bound_model)));
+
+  if (priv->selected_index == selected_index)
+    return;
+
+  priv->selected_index = selected_index;
+  update (self);
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_SELECTED_INDEX]);
+}
+
+/**
+ * hdy_combo_row_get_use_subtitle:
+ * @self: a #GtkListBoxRow
+ *
+ * Gets whether the current value of @self should be displayed as its subtitle.
+ *
+ * Returns: whether the current value of @self should be displayed as its subtitle
+ *
+ * Since: 0.0.10
+ */
+gboolean
+hdy_combo_row_get_use_subtitle (HdyComboRow *self)
+{
+  HdyComboRowPrivate *priv;
+
+  g_return_val_if_fail (HDY_IS_COMBO_ROW (self), FALSE);
+
+  priv = hdy_combo_row_get_instance_private (self);
+
+  return priv->use_subtitle;
+}
+
+/**
+ * hdy_combo_row_set_use_subtitle:
+ * @self: a #HdyComboRow
+ * @use_subtitle: %TRUE to set the current value as the subtitle
+ *
+ * Sets whether the current value of @self should be displayed as its subtitle.
+ *
+ * If %TRUE, you should not access HdyActionRow:subtitle.
+ *
+ * Since: 0.0.10
+ */
+void
+hdy_combo_row_set_use_subtitle (HdyComboRow *self,
+                                gboolean     use_subtitle)
+{
+  HdyComboRowPrivate *priv;
+
+  g_return_if_fail (HDY_IS_COMBO_ROW (self));
+
+  priv = hdy_combo_row_get_instance_private (self);
+
+  use_subtitle = !!use_subtitle;
+
+  if (priv->use_subtitle == use_subtitle)
+    return;
+
+  priv->use_subtitle = use_subtitle;
+  update (self);
+  if (!use_subtitle)
+    hdy_action_row_set_subtitle (HDY_ACTION_ROW (self), NULL);
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_USE_SUBTITLE]);
+}
+
+/**
+ * hdy_combo_row_set_get_name_func:
+ * @self: a #HdyComboRow
+ * @get_name_func: (nullable): a function that creates names for items, or %NULL
+ *   in case you also passed %NULL as @model
+ * @user_data: user data passed to @get_name_func
+ * @user_data_free_func: function for freeing @user_data
+ *
+ * Sets a closure to convert items into names. See HdyComboRow:use-subtitle.
+ *
+ * Since: 0.0.10
+ */
+void
+hdy_combo_row_set_get_name_func (HdyComboRow            *self,
+                                 HdyComboRowGetNameFunc  get_name_func,
+                                 gpointer                user_data,
+                                 GDestroyNotify          user_data_free_func)
+{
+  HdyComboRowPrivate *priv;
+
+  g_return_if_fail (HDY_IS_COMBO_ROW (self));
+
+  priv = hdy_combo_row_get_instance_private (self);
+
+  get_name_free (priv->get_name);
+  priv->get_name = g_new0 (HdyComboRowGetName, 1);
+  priv->get_name->func = get_name_func;
+  priv->get_name->func_data = user_data;
+  priv->get_name->func_data_destroy = user_data_free_func;
+}
+
 /**
  * hdy_enum_value_row_name:
  * @value: the value from the enum from which to get a name
